﻿# Scriptpath auslesen / prüfen ob Shell oder ISE
if([string]::IsNullOrEmpty($PSScriptRoot)){
    $Scriptpath = ($psISE.CurrentFile.FullPath -replace $psISE.CurrentFile.DisplayName).TrimEnd("\")
} else{
    $Scriptpath = $PSScriptRoot
}

# Setzen der benötigten Infos
# $true -> AuditMode ist aktiv
# $false -> EnforcmentMode ist aktiv
$AuditOnly = $true

# Distinguished Name der Organisatiosneinheit, wo die Gruppen erstellt
# werden
$GroupOU = "OU=Groups,OU=JansOU,DC=ad,DC=adsvc,DC=org"

# Gruppennamen deklarieren
$GALUsersGroupName = "G-AppLocker_Users"
$DLALUsersGroupName = "DL-AppLocker_Users-Deny"
$GALAdminsGroupName = "G-AppLocker_Admins"
$DLALAdminsGroupName = "DL-AppLocker_Admins-Allow"

# Verzeichnispfade setzen
$XMLPVorlagen = Join-Path $Scriptpath `
    -ChildPath "AppLocker\AppLocker_Vorlage"
$XMLPImport = Join-Path $Scriptpath `
    -ChildPath "AppLocker\AppLocker_Import"
$GPOPImport = Join-Path -Path $XMLPImport `
    -ChildPath "C_AppLocker_Allow_Base_Enforce"

# Auslesen der evtl. bereits vorhandenen Grupoen
$objGALUsers = Get-ADGroup -Filter 'name -eq $GALUsersGroupName'
$objDLALUsers = Get-ADGroup -Filter 'name -eq $DLALUsersGroupName'
$objGALAdmins = Get-ADGroup -Filter 'name -eq $GALAdminsGroupName'
$objDLALAdmins = Get-ADGroup -Filter 'name -eq $DLALAdminsGroupName'

# Auslesen der aktuellen Domain des Computers
$objADDomain = Get-ADDomain -Current LocalComputer

# Einlesen aller vorhandenen GPO
$objAllGPO = Get-GPO -All

# Anlegen der Gruppen sofern nicht bereits vorhanden
if($null -eq $objGALUsers){
    Write-Host "Erstelle globale Gruppe `"$GALUsersGroupName`""
    $objGALUsers = New-ADGroup -Name $GALUsersGroupName `
        -DisplayName $GALUsersGroupName `
        -Description "Gruppe aller Applocker Benutzer" `
        -GroupCategory Security `
        -GroupScope Global `
        -Path $GroupOU `
        -PassThru
}
if($null -eq $objDLALUsers){
    Write-Host "Erstelle domänen lokale Gruppe `"$DLALUsersGroupName`""
    $objDLALUsers = New-ADGroup -Name $DLALUsersGroupName `
        -DisplayName $DLALUsersGroupName `
        -Description "Applocker Deny Gruppe" `
        -GroupCategory Security `
        -GroupScope DomainLocal `
        -Path $GroupOU `
        -PassThru
}
if($null -eq $objGALAdmins){
    Write-Host "Erstelle globale Gruppe `"$GALAdminsGroupName`""
    $objGALAdmins = New-ADGroup -Name $GALAdminsGroupName `
        -DisplayName $GALAdminsGroupName `
        -Description "Gruppe der Administratoren" `
        -GroupCategory Security `
        -GroupScope Global `
        -Path $GroupOU `
        -PassThru
}
if($null -eq $objDLALAdmins){
    Write-Host "Erstelle domänen lokale Gruppe `"$DLALAdminsGroupName`""
    $objDLALAdmins = New-ADGroup -Name $DLALAdminsGroupName `
        -DisplayName $DLALAdminsGroupName `
        -Description "Applocker Alow Gruppe" `
        -GroupCategory Security `
        -GroupScope DomainLocal `
        -Path $GroupOU `
        -PassThru
}

# Aufnehmen der globalen Gruppen in die domänen lokalen Gruppen 
Add-ADGroupMember -Identity $objDLALUsers `
    -Members $objGALUsers
Add-ADGroupMember -Identity $objDLALAdmins `
    -Members $objGALAdmins

# Die Accounts der User / Admins müssen händisch in die globalen
# Gruppen aufgenommen werden

if(-not (Test-Path -Path $XMLPImport -PathType Container)){
    New-Item -Path $XMLPImport `
        -ItemType Directory |
            Out-Null
}

# Einlesen der Applocker Vorlagen XMLs
$objALVXmls = Get-ChildItem -Path $XMLPVorlagen `
    -Filter *.xml

# Jede XML einlesen und die Platzhalter durch aktuelle Werte
# erstzen sowie schreiben der späteren XML Import Files. 
# <SID Admin Gruppe> -> SID der neuen / ausgelesenen Admin Gruppe
# <SID User Gruppe> -> SID der neuen / ausgelesenen User Gruppe
# <Domain FQDN> -> DNS Name der aktuellen Domäne
foreach($x in $objALVXmls){
    $tmpContent = @()
    $objSRead = [System.IO.StreamReader]::new($x.FullName)
    while($objSRead.EndOfStream -eq $false){
        $l = $objSRead.ReadLine()
        if($l -match "<SID Admin Gruppe>"){
            $l = $l -replace "<SID Admin Gruppe>", $objDLALAdmins.SID
        }
        if($l -match "<SID User Gruppe>"){
            $l = $l -replace "<SID User Gruppe>", $objDLALUsers.SID
        }
        if($l -match "<Domain FQDN>"){
            $l = $l -replace "<Domain FQDN>", $objADDomain.DNSRoot
        }
        if($AuditOnly -eq $true){
            if($l -match "EnforcementMode=`"Enabled`""){
                $l = $l -replace "EnforcementMode=`"Enabled`"", "EnforcementMode=`"AuditOnly`""
            }
        }
        $tmpContent += $l
    }
    $objSRead.Close()
    $IFName = Join-Path -Path $XMLPImport `
        -ChildPath $x.Name
    Set-Content -Path $IFName `
        -Value $tmpContent `
        -Encoding UTF8 `
        -Confirm:$false `
        -Force
}

# Einlesen der Applocker Import XMLs
$objALIXmls = Get-ChildItem -Path $XMLPImport `
    -Filter *.xml

# Jede XML einlesen und prüfen, ob ein passendes GPO vorhanden ist
# und ggfs. Erstellung des GPO. Sofern es sich um das Base GPO handelt,
# wird dieses importiert (Hier wird der Application Identity Service
# auf "Automatisch" konfiguriert; Das Applocker Eventlog auf 10 MB
# erhöht; Der ActiveSetup Key für "ie4uinit.exe" gelöscht*). Ebenfalls
# wird jeweils das entsprechende Import XML importiert.
# *
#  -> Computer Configuration -> Policies -> Windows Settings -> Security
#  Settings -> System Services -> Application Identity -> Autoamtic
#  -> Computer Configuration -> Preferences -> Windows Settings ->
#  Registry
foreach($x in $objALIXmls){
    $curobjGPO = $objAllGPO |
        Where-Object { $_.DisplayName -match $x.BaseName }
    if($null -eq $curobjGPO){
        Write-Host "Erstelle GPO `"$($x.BaseName)`""
        $curobjGPO = New-GPO -Name $x.BaseName `
            -Comment (-join("Applocker: ", $x.BaseName)) `
            -Domain $objADDomain.DNSRoot            
    }
    if($curobjGPO.DisplayName -match "C_AppLocker_Allow_Base_Enforce"){
        Write-Host "Importiere GPO `"$($x.BaseName)`""
        Import-GPO -TargetGuid $curobjGPO.Id `
            -BackupGpoName "C_AppLocker_Allow_Base_Enforce" `
            -Path $GPOPImport `
            -Domain $objADDomain.DNSRoot |
                Out-Null
    }
    $curobjGPO.GpoStatus = "UserSettingsDisabled"
    $curLDAPPath = -join(
        "LDAP://",
        $objADDomain.DNSRoot,
        "/CN={",
        $curobjGPO.Id,
        "},CN=Policies,CN=System,",
        $objADDomain.DistinguishedName
    )
    Write-Host "Importiere AppLocker XML `"$($x.Name)`" in GPO `"$($curobjGPO.DisplayName)`""
    Set-AppLockerPolicy -XmlPolicy $x.FullName `
        -Ldap $curLDAPPath

    # Hier wäre der Part zu ergänzen, um die Policies zu verlinken
    # In $OUName sollte der Name der OU, wo die Policies verknüpft
    # werden sollen
    <#
        $OUName = "DATEVTDPolicy"
        $objRDSHOU = Get-ADOrganizationalUnit -Filter 'name -eq $OUName'
        New-GPLink -Guid $curobjGPO.Id `
            -Target $objRDSHOU.DistinguishedName `
            -LinkEnabled Yes `
            -Domain $objADDomain.DNSRoot       
    #>
}